package sample;

import java.util.*;
import org.aspectj.lang.reflect.*;
import org.aspectj.lang.*;

abstract aspect AspectBasedMethodInterceptor {
	pointcut allCalls() : execution(* *.* (..))
		&& !within(sample.ComponentTestCase || sample.AspectBasedMethodInterceptor+);
		
	pointcut allCallsWithoutException() : allCalls() && !execution(* *.* (..) throws *);
			
	Object around() : allCallsWithoutException()
	{
		System.out.println("allCalls -> "+thisJoinPoint.toString());
		Object r  = mockCall(thisJoinPoint);
		
		if (r != null)
			return r;
		else
		{
			return proceed();
		}
	}

	protected Object mockCall(JoinPoint jp)
	{
		String className = jp.getSignature().getDeclaringType().getName();
		Object receiver = jp.getThis();
		if (receiver != null)
			className = receiver.getClass().getName();
		String methodName = jp.getSignature().getName();
		Object returnValue = ComponentTestCase.getMockReturnValue(className, methodName);
		if (returnValue != null)
		{
			ComponentTestCase.indicateCalled(className, methodName, getArguments(jp));

		}
		return returnValue;

	}

	protected Object throwExceptionIfNeeded(JoinPoint jp) throws Throwable
	{
		String className = jp.getSignature().getDeclaringType().getName();
		String methodName = jp.getSignature().getName();
		Throwable t = ComponentTestCase.getMockException(className, methodName);
		if (t != null)
		{
			ComponentTestCase.indicateCalled(className, methodName, getArguments(jp));
			throw t;
		}
		return mockCall(jp);
	}

	protected Hashtable getArguments(JoinPoint jp)
	{
		Hashtable arguments = new Hashtable();
		Object[] argumentValues = jp.getArgs();
		String[] argumentNames = ((CodeSignature)jp.getSignature()).getParameterNames();
		for (int i = 0; i < argumentValues.length; i++)
		{
			if (argumentValues[i] != null)
				arguments.put(argumentNames[i], argumentValues[i]);
		}
		return arguments;
	}
}
